@@ -11,87 +11,106 @@ import json
 import io, zipfile
 import sys
 import logging
+
 library_tools = LibraryTools()
 env_path = os.getenv("HOME") + "/.config/template_library"
+templates_types = {"data_types": 1, "artifact_types": 2, "capability_types": 3, "requirement": 4, "relationship_types": 5, "interface_types": 6, "node_types": 7,
+                   "group": 8, "policy_types": 9, "csar": 10, "other": 11}
+templates_type_names = {"data_types": "DataType", "artifact_types": "ArtifactType", "capability_types": "CapabilityType",
+                        "requirement": "Requirement", "relationship_types": "RelationshipType",
+                        "interface_types": "InterfaceType", "node_types": "NodeType",
+                   "group": "Group", "policy_types": "PolicyType", "csar": "Csar", "other": "Other"}
 
 
-def validate_content(node_type_path, args, api_address, headers):
-    files = {}
+def add_entity_template(node_type_path, args, api_address, headers):
+    files = []
     if os.path.isdir(node_type_path):
         node_file_list = []
         for node_subdir, node_dirs, node_files in os.walk(node_type_path):
             for file in node_files:
                 node_file_list.append(file)
+                if ".yml" in str(file):
+                    files.append(('implementation_file', open(args.path + "/files/" + str(file), 'rb')))
+                if ".tosca" in str(file):
+                    files.append(('template_file', open(args.path + "/" + str(file), 'rb')))
+
         implementation = any('.yml' in x for x in node_file_list)
-        definition = 'NodeType.tosca' in node_file_list
+        definition = any('.tosca' in x for x in node_file_list)
         if not implementation:
             print("Implementation missing for " + node_type_path + "\n")
-        else:
-            impl = {
-                "implementation_file": ("create.yml", open(args.path + "/files/create.yml", 'rb'), 'application-type')
-            }
-            files.update(impl)
         if not definition:
             print("Definition missing for " + node_type_path + "\n")
-        else:
-            defi = {
-                "template_file": ("NodeType.tosca", open(args.path + "/NodeType.tosca", 'rb'), 'application-type')
-            }
-            files.update(defi)
 
-        api_url = api_address + "/api/templates"
-        name, group = get_name_type(node_type_path)
-        access = json.loads(args.public_access.lower())
-        # TODO: if template already exists and ou want to make in public as well you have to give a new name
-        data = {
-            "shorthand_name": args.name,
-            "type_uri": group,
-            "template_type_id": 7,
-            "public_access": str(access)
-        }
+        name, group, template_type = get_name_type(node_type_path)
+        template_id = add_template(api_address, args, headers, group, templates_types[template_type])
 
-        headers = authorization(headers)
-
-        try:
-            response = requests.post(api_url, data=str(data), headers=headers)
-            response.raise_for_status()
-            template_id = json.loads(response.content.decode("utf-8"))["object_id"]
-        except requests.HTTPError as e:
-            logging.debug(e)
-            print(e.response.content.decode("utf-8"))
-            if e.response.status_code == 406:
-                api_url = api_address + "/api/templates/name/" + args.name
-                try:
-                    response = requests.get(api_url, headers=headers)
-                    template_id = json.loads(response.content.decode("utf-8"))["id"]
-                    response.raise_for_status()
-                except requests.HTTPError as e:
-                    logging.debug(e)
-                    print(e.response.content.decode("utf-8"))
-
-        api_url = api_address + "/api/versions"
-
-        data = {
-            "template_id": template_id,
-            "version": args.version
-        }
-
-        headers.pop("Content-type")
-        try:
-            response = requests.Session().post(api_url, headers=headers, data=data, files=files)
-            response.raise_for_status()
-            print("New template version created!")
-        except requests.HTTPError as e:
-            logging.debug(e)
-            print(e.response.content.decode("utf-8"))
-            exit(1)
+        add_version(api_address, args, headers, files, template_id)
 
     elif tarfile.is_tarfile(os.path.normpath(node_type_path)) or zipfile.is_zipfile(os.path.normpath(node_type_path)):
-        fileobj = open(node_type_path, 'rb')
-        zip_file = {"archive": ("csar.zip", fileobj)}
-        files.update(zip_file)
-        # TODO:
-        print("You can't upload zip files yet.")
+        print("Entity template can only be uploaded as a directory.")
+
+
+def add_service_template(node_type_path, args, api_address, headers):
+    files = []
+    if os.path.isdir(node_type_path):
+        print("Service template can only be uploaded in tar or zip format.")
+    elif tarfile.is_tarfile(os.path.normpath(node_type_path)) or zipfile.is_zipfile(os.path.normpath(node_type_path)):
+        zip_file = ('template_file', open(node_type_path, 'rb'))
+        files.append(zip_file)
+
+        template_id = add_template(api_address, args, headers, "node.blueprint", 10)
+        add_version(api_address, args, headers, files, template_id)
+
+
+def add_version(api_address, args, headers, files, template_id):
+    api_url = api_address + "/api/versions"
+
+    data = {
+        "template_id": template_id,
+        "version": args.version
+    }
+
+    headers.pop("Content-type")
+    try:
+        response = requests.Session().post(api_url, headers=headers, data=data, files=files)
+        response.raise_for_status()
+        print("New template version created!")
+    except requests.HTTPError as e:
+        logging.debug(e)
+        print(e.response.content.decode("utf-8"))
+        exit(1)
+
+
+def add_template(api_address, args, headers, group, node_type_id):
+    api_url = api_address + "/api/templates"
+    access = args.public_access
+    # TODO: if template already exists and you want to make in public as well you have to give a new name
+    data = {
+        "shorthand_name": args.name,
+        "type_uri": group,
+        "template_type_id": node_type_id,
+        "public_access": str(access)
+    }
+
+    headers = authorization(headers)
+
+    try:
+        response = requests.post(api_url, data=str(data), headers=headers)
+        response.raise_for_status()
+        template_id = json.loads(response.content.decode("utf-8"))["object_id"]
+    except requests.HTTPError as e:
+        logging.debug(e)
+        print(e.response.content.decode("utf-8"))
+        if e.response.status_code == 406:
+            api_url = api_address + "/api/templates/name/" + args.name
+            try:
+                response = requests.get(api_url, headers=headers)
+                template_id = json.loads(response.content.decode("utf-8"))["id"]
+                response.raise_for_status()
+            except requests.HTTPError as e:
+                logging.debug(e)
+                print(e.response.content.decode("utf-8"))
+    return template_id
 
 
 def download_template(args, api_address, headers):
@@ -111,11 +130,12 @@ def download_template(args, api_address, headers):
 
 
 def list_templates(args, api_address, headers):
-    access = json.loads(args.public_access.lower())
+    access = args.public_access
     headers = authorization(headers)
 
     if access:
-        api_url = api_address + "/api/templates"
+        #api_url = api_address + "/api/templates"
+        api_url = api_address + "/api/templates/template_type/" + "10"
 
     else:
         file = open(env_path + "/.username", "r")
@@ -196,8 +216,9 @@ def get_definition(node_type_path):
 def get_name_type(node_type_path):
     node_type_path += "/" + get_definition(node_type_path)
     data_dict = library_tools.implementation_to_dictionary(node_type_path)
-    group, name = list(data_dict['node_types'].keys())[0].rsplit('.', 1)
-    return name, group
+    template_type = list(data_dict.keys())[1]
+    group, name = list(data_dict[template_type].keys())[0].rsplit('.', 1)
+    return name, group, template_type
 
 
 def store_true():
@@ -230,36 +251,67 @@ def main():
 
     command_options = parser.add_subparsers(dest='command')
 
-    templates = command_options.add_parser('template', help='Edit a template.')
-    template_options = templates.add_subparsers(dest='template_options')
-    save = template_options.add_parser('save', help='Save a template to database.')
-    get = template_options.add_parser('get', help='Get a template from database.')
-    list_parser = template_options.add_parser('list', help='List templates from database.')
-    version_parser = template_options.add_parser('version', help='Version options for templates.')
+    service_templates = command_options.add_parser('service-template', help='Edit a service template.')
+    service_template_options = service_templates.add_subparsers(dest='template_options')
+    save_service = service_template_options.add_parser('save', help='Save a template to database.')
+    get_service = service_template_options.add_parser('get', help='Get a template from database.')
+    list_service_parser = service_template_options.add_parser('list', help='List templates from database.')
+    version_service_parser = service_template_options.add_parser('version', help='Version options for templates.')
 
-    version_options = version_parser.add_subparsers(dest='version_options')
-    list_version = version_options.add_parser('list', help='List versions of a template from database.')
-    list_version_group = list_version.add_mutually_exclusive_group(required=True)
-    list_version_group.add_argument('--template_id', help='Id of the template.', dest='template_id')
-    list_version_group.add_argument('--template_name', help='Name of the template.', dest='template_name')
+    version_service_options = version_service_parser.add_subparsers(dest='version_options')
+    list_service_version = version_service_options.add_parser('list', help='List versions of a template from database.')
+    list_service_version_group = list_service_version.add_mutually_exclusive_group(required=True)
+    list_service_version_group.add_argument('--template_id', help='Id of the template.', dest='template_id')
+    list_service_version_group.add_argument('--template_name', help='Name of the template.', dest='template_name')
 
-    save.add_argument('--name', help='Name of the template.', dest='name', required=True)
-    save.add_argument('--path', help='Path to the template code or desired template location.', dest='path',
-                           required=True)
-    save.add_argument('--public_access', help='Is template available to the public or just this account',
-                           dest='public_access', required=True)
-    save.add_argument('--version', help='Version of the template to upload/download',
-                           dest='version', required=True)
+    save_service.add_argument('--name', help='Name of the template.', dest='name', required=True)
+    save_service.add_argument('--path', help='Path to the template code or desired template location.', dest='path',
+                             required=True)
+    save_service.add_argument('--public_access', help='Is template available to the public or just this account',
+                             dest='public_access', action="store_true", default=False)
+    save_service.add_argument('--version', help='Version of the template to upload/download',
+                             dest='version', required=True)
 
-    get.add_argument('--version_id', help='Id of the template.', dest='version_id', required=True)
-    get.add_argument('--path', help='Path to the template code or desired template location.', dest='path',
-                           required=True)
-    get.add_argument('--public_access', help='Is template available to the public or just this account',
-                           dest='public_access', required=True)
+    get_service.add_argument('--version_id', help='Id of the template.', dest='version_id', required=True)
+    get_service.add_argument('--path', help='Path to the template code or desired template location.', dest='path',
+                            required=True)
+    get_service.add_argument('--public_access', help='Is template available to the public or just this account',
+                            dest='public_access', action="store_true", default=False)
 
-    list_parser.add_argument('--name', help='Name of the template.', dest='name')
-    list_parser.add_argument('--public_access', help='Is template available to the public or just this account',
-                     dest='public_access', required=True)
+    list_service_parser.add_argument('--name', help='Name of the template.', dest='name')
+    list_service_parser.add_argument('--public_access', help='Is template available to the public or just this account',
+                                    dest='public_access', action="store_true", default=False)
+
+    entity_templates = command_options.add_parser('entity-template', help='Edit an entity template.')
+    entity_template_options = entity_templates.add_subparsers(dest='template_options')
+    save_entity = entity_template_options.add_parser('save', help='Save a template to database.')
+    get_entity = entity_template_options.add_parser('get', help='Get a template from database.')
+    list_entity_parser = entity_template_options.add_parser('list', help='List templates from database.')
+    version_entity_parser = entity_template_options.add_parser('version', help='Version options for templates.')
+
+    version_entity_options = version_entity_parser.add_subparsers(dest='version_options')
+    list_entity_version = version_entity_options.add_parser('list', help='List versions of a template from database.')
+    list_entity_version_group = list_entity_version.add_mutually_exclusive_group(required=True)
+    list_entity_version_group.add_argument('--template_id', help='Id of the template.', dest='template_id')
+    list_entity_version_group.add_argument('--template_name', help='Name of the template.', dest='template_name')
+
+    save_entity.add_argument('--name', help='Name of the template.', dest='name', required=True)
+    save_entity.add_argument('--path', help='Path to the template code or desired template location.', dest='path',
+                      required=True)
+    save_entity.add_argument('--public_access', help='Is template available to the public or just this account',
+                      dest='public_access', action="store_true", default=False)
+    save_entity.add_argument('--version', help='Version of the template to upload/download',
+                      dest='version', required=True)
+
+    get_entity.add_argument('--version_id', help='Id of the template.', dest='version_id', required=True)
+    get_entity.add_argument('--path', help='Path to the template code or desired template location.', dest='path',
+                     required=True)
+    get_entity.add_argument('--public_access', help='Is template available to the public or just this account',
+                     dest='public_access', action="store_true", default=False)
+
+    list_entity_parser.add_argument('--name', help='Name of the template.', dest='name')
+    list_entity_parser.add_argument('--public_access', help='Is template available to the public or just this account',
+                             dest='public_access', action="store_true", default=False)
 
     setup = command_options.add_parser('setup', help='Setup client variables.')
 
@@ -272,7 +324,8 @@ def main():
     # TODO: we could also have a validate-model
     create_model = command_options.add_parser('create-model', help='Initialize a model directory and files.')
 
-    create_blueprint = command_options.add_parser('create-blueprint', help='Initialize a blueprint directory and files.')
+    create_blueprint = command_options.add_parser('create-blueprint',
+                                                  help='Initialize a blueprint directory and files.')
 
     parser.add_argument("-v", "--verbose", help="Increase output verbosity", action="store_true")
 
@@ -359,9 +412,13 @@ def main():
 
     if args.command == "create-model":
         model_name = input("Model name:")
+        model_type = input("Model type:")
+        if model_type not in templates_types:
+            print("Template type doesn't exist. Choose from: " + str(templates_types.keys()))
+            exit(1)
         try:
             os.mkdir(model_name)
-            open(model_name + "/NodeType.tosca", 'a').close()
+            open(model_name + "/" + templates_type_names[model_type] + ".tosca", 'a').close()
 
             model_name += "/files"
             os.mkdir(model_name)
@@ -378,9 +435,9 @@ def main():
         except Exception as e:
             logging.debug(e)
 
-    if args.command == "template":
+    if args.command == "entity-template":
         if args.template_options == "save":
-            validate_content(args.path, args, api_address, headers)
+            add_entity_template(args.path, args, api_address, headers)
         if args.template_options == "get":
             download_template(args, api_address, headers)
         if args.template_options == "list":
@@ -388,3 +445,14 @@ def main():
         if args.template_options == "version":
             if args.version_options == "list":
                 list_versions(args, api_address, headers)
+
+    if args.command == "service-template":
+        if args.template_options == "save":
+            add_service_template(args.path, args, api_address, headers)
+        if args.template_options == "get":
+            download_template(args, api_address, headers)
+        if args.template_options == "list":
+            list_templates(args, api_address, headers)
+        if args.template_options == "version":
+            if args.version_options == "list":
+                list_versions(args, api_address, headers)
\ No newline at end of file
